/***********************************************************************
 *    DecoEmojiListDao
 *
 * [File]
 *     DecoEmojiListDao.java
 *
 *     Copyright (C) 2014 OMRON SOFTWARE Co., Ltd.  All Rights Reserved.
 *
 * [Description]
 *     DecoEmojiList Data Access Object Class.
 ************************************************************************/
package com.emojisampleapplication;

import java.io.File;
import java.util.ArrayList;
import java.util.List;

import jp.co.omronsoft.android.emoji.EmojiAssist;
import android.content.ContentValues;
import android.content.Context;
import android.database.Cursor;
import android.database.SQLException;
import android.database.sqlite.SQLiteDatabase;
import android.database.sqlite.SQLiteException;
import android.database.sqlite.SQLiteOpenHelper;
import android.util.Log;
import android.net.Uri;

import com.emojisampleapplication.constants.Const;

import jp.co.omronsoft.android.decoemojimanager_docomo.interfacedata.DecoEmojiCategoryInfo;
import jp.co.omronsoft.android.decoemojimanager_docomo.interfacedata.DecoEmojiContract;
import jp.co.omronsoft.android.decoemojimanager_docomo.interfacedata.DecoEmojiContract.DecoEmojiInfoColumns;

/**
 * 一覧表示制御アプリ 入力履歴テーブル操作用DAOクラス.
 */
public class DecoEmojiListDao {

    /** For Debug Class Name. */
    private static final String CLASS_NAME = "DecoEmojiListDao#";

    /** DB操作用クラスを表すフィールド. */
    private SQLiteDatabase mDatabase = null;

    /** ローカルコンテキスト(DecoEmojiList)を表すフィールド. */
    private Context mLocalContext = null;

    /** 入力履歴テーブルのレコードを表すフィールド. */
    private List<EmojiAssist.DecoEmojiTextInfo> mDecoHistories;

    /** 入力履歴テーブルのレコードが存在するかを表すフィールド. */
    private boolean mIsExistDecoHistory = false;

    /** 1画面に表示するデコ絵文字数を表すフィールド. */
    private int mDecoEmojiCount = 0;

    /** カテゴリ情報のリストを表すフィールド. */
    private ArrayList<DecoEmojiCategoryInfo> mCategoryInfoList;

    /** デコ絵文字一覧のリストを表すフィールド. */
    private ArrayList<EmojiAssist.DecoEmojiTextInfo> mCurrentDecoEmojiList;

    /** カテゴリ内に表示する絵文字数を表すフィールド. */
    private int mDecoEmojiListCount = 0;

    /** DB操作用クラスを宣言する. */
    private static class DatabaseHelper extends SQLiteOpenHelper {

        /** コンストラクタ. */
        DatabaseHelper(Context context) {
            super(context, Const.DB_NAME, null, Const.DB_VERSION);
        }

        /** @see android.database.sqlite.SQLiteOpenHelper#onCreate */
        @Override
        public void onCreate(SQLiteDatabase db) {
            try {
                db.execSQL(Const.SQL_CREATE_TABLE_DECO);
                db.execSQL(Const.SQL_CREATE_TABLE_UNICODE);
            } catch (SQLException e) {
                e.printStackTrace();
            }
        }

        /** @see android.database.sqlite.SQLiteOpenHelper#onUpgrade */
        @Override
        public void onUpgrade(SQLiteDatabase db, int oldVersion, int newVersion) {
            try {
                db.execSQL(Const.SQL_DROP_TABLE_DECO);
                db.execSQL(Const.SQL_DROP_TABLE_UNICODE);
            } catch (SQLException e) {
                e.printStackTrace();
            }
            onCreate(db);
        }
    }

    /** コンストラクタ. */
    public DecoEmojiListDao(Context context) {
        mLocalContext = context;
        openHistories();
    }

    /** DBの接続を行う. */
    private void openHistories() {
        closeHistories();
        DatabaseHelper dbHelper = new DatabaseHelper(mLocalContext);
        mDatabase = dbHelper.getWritableDatabase();
    }

    /** DBの切断を行う. */
    void closeHistories() {
        if (mDatabase != null) {
            mDatabase.close();
            mDatabase = null;
        }
    }

    /**
     * 入力履歴が存在するかを取得する.
     * 
     * @return 存在フラグ(true:存在する/false:存在しない)
     */
    boolean isExistHistories() {
        return mIsExistDecoHistory;
    }

    /**
     * 初期処理を行う.
     * 
     * @param pageSize
     *            1ページに表示するデコ絵文字数
     */
    void init(int decoEmojiCount, int emojiType) {
        if (Const.DEBUG) {
            Log.d(Const.TAG, CLASS_NAME + "init() start. decoEmojiCount[" + decoEmojiCount + "]");
        }

        mDecoEmojiCount = decoEmojiCount;
        mDecoEmojiListCount = 0;
        if (mDecoEmojiCount != 0) {
            int rowCount;
            String kind = String.valueOf(DecoEmojiContract.KIND_SQ20);
            // 対象絵文字タイプの値を絵文字種別の値に変換する
            kind = DecoEmojiContract.makeStringEmojiKind(emojiType);
            Uri decoinfolistUri;
            decoinfolistUri = DecoEmojiContract.CONTENT_DECOINFOLIST_URI;
            String column[] = {DecoEmojiInfoColumns.DECOEMOJI_ID,
                               DecoEmojiInfoColumns.URI};
            String selection = DecoEmojiInfoColumns.HISTORY_CNT + " > " + Const.VALUE_0
                                    + " and " + DecoEmojiInfoColumns.KIND + " in ( " + kind + " ) ";
            String orderBy = DecoEmojiInfoColumns.HISTORY_CNT + " desc";
            // 利用履歴情報の同期(履歴情報の取得)
            Cursor cursor = mLocalContext.getContentResolver()
                    .query(decoinfolistUri, column, selection, null, orderBy);

            if (cursor != null) {
                rowCount = cursor.getCount();
                mDecoEmojiListCount = rowCount;
                cursor.moveToFirst();
                for (int i = 0; i < rowCount; i++) {
                    String uri = cursor.getString(Const.COLUMN_INDEX_URI);
                    File file = new File(uri);
                    if (file.exists()) {
                        mIsExistDecoHistory = true;
                        break;
                    }
                    cursor.moveToNext();
                }
                cursor.close();
            }
        }

        if (Const.DEBUG) {
            Log.d(Const.TAG, CLASS_NAME + "init() end.");
        }
    }

    /**
     * 入力履歴テーブルのレコードを取得する.
     * 
     * @param pageNum
     *            取得するページ数
     * @return 入力履歴テーブルのレコード
     */
    List<EmojiAssist.DecoEmojiTextInfo> loadHistories(int pageNum, int startNum, int emojiType) {
        if (Const.DEBUG) {
            Log.d(Const.TAG, CLASS_NAME + "loadHistories() start. pageNum[" + pageNum + "]");
        }

        int rowCount = 0;

        // 取得する入力履歴の範囲を算出する(limitからoffset分取得する)
        int offset = startNum;
        int limit = Const.DB_MAX_HISTORY_RECORD;

        // limitがマイナスとなる検索はエラーとなる為
        if (limit < 0) {
            if (Const.DEBUG) {
                Log.d(Const.TAG, CLASS_NAME + "loadHistories() end. 'limit' Is Minus.");
            }
            return null;
        }

        mDecoHistories = new ArrayList<EmojiAssist.DecoEmojiTextInfo>();
        String kind = String.valueOf(DecoEmojiContract.KIND_SQ20);
        // 対象絵文字タイプの値を絵文字種別の値に変換する
        kind = DecoEmojiContract.makeStringEmojiKind(emojiType);
        Uri decoinfolistUri;
        decoinfolistUri = DecoEmojiContract.CONTENT_DECOINFOLIST_URI;
        // 詳細取得方式のサンプルコード：URI、絵文字の種別、POP対応の情報を取得する
        String column[] = {DecoEmojiInfoColumns.DECOEMOJI_ID,
                           DecoEmojiInfoColumns.URI,
                           DecoEmojiInfoColumns.WIDTH,
                           DecoEmojiInfoColumns.HEIGHT,
                           DecoEmojiInfoColumns.KIND,
                           DecoEmojiInfoColumns.DECOME_POP_FLAG,
                           DecoEmojiInfoColumns.TAGS};
        String selection = DecoEmojiInfoColumns.HISTORY_CNT + " > "
                + Const.VALUE_0 + " and " + DecoEmojiInfoColumns.KIND
                + " in ( " + kind + " ) ";

        String orderBy = DecoEmojiInfoColumns.HISTORY_CNT + " desc limit "
                + String.valueOf(offset) + "," + String.valueOf(limit);
        Cursor cursor = mLocalContext.getContentResolver()
                .query(decoinfolistUri, column, selection, null, orderBy);

        if (cursor == null) {
            return null;
        }

        rowCount = cursor.getCount();

        if (rowCount == 0) {
            cursor.close();

            if (Const.DEBUG) {
                Log.d(Const.TAG, CLASS_NAME + "loadHistories() end. No Histories.");
            }
            return null;
        }

        cursor.moveToFirst();
        for (int i = 0; i < rowCount; i++) {
            String uri = cursor.getString(Const.COLUMN_INDEX_URI);
            File file = new File(uri);
            if (file.exists()) {
                EmojiAssist.DecoEmojiTextInfo decoEmojiTextInfo = new EmojiAssist.DecoEmojiTextInfo();
                decoEmojiTextInfo.setUri(uri);
                decoEmojiTextInfo.setWidth(cursor.getInt(2));
                decoEmojiTextInfo.setHeight(cursor.getInt(3));
                decoEmojiTextInfo.setKind(cursor.getInt(4));
                decoEmojiTextInfo.setPop(cursor.getInt(5));
                if (!cursor.getString(6).equals("")) {
                    decoEmojiTextInfo.setCustomTags(true);
                }
                mDecoHistories.add(decoEmojiTextInfo);
            }
            cursor.moveToNext();
        }
        cursor.close();

        if (Const.DEBUG) {
            Log.d(Const.TAG, CLASS_NAME + "loadHistories() end.");
        }

        return mDecoHistories;
    }

    /**
     * Unicode絵文字入力履歴が存在するかを取得する.
     * 
     * @return 存在フラグ(true:存在する/false:存在しない)
     */
    boolean isExistHistoriesUnicode() {

        boolean isExist = false;

        SQLiteDatabase con = mDatabase;

        Cursor cursor = con.query(Const.TABLE_NAME_UNICODE, new String[] {
                Const.COLUMN_HISTORY_ID, Const.COLUMN_CODE }, null, null, null, null, null);
        int rowCount = cursor.getCount();
        cursor.close();

        if (rowCount > 0) {
            isExist = true;
        }

        return isExist;
    }

    /**
     * Unicode絵文字入力履歴テーブルのレコードを取得する.
     * 
     * @return 入力履歴テーブルのレコード
     */
    List<EmojiAssist.DecoEmojiTextInfo> loadHistoriesUnicode() {
        if (Const.DEBUG) {
            Log.d(Const.TAG, CLASS_NAME + "loadHistories() start. ");
        }

        int rowCount = 0;

        SQLiteDatabase con = mDatabase;

        mDecoHistories = new ArrayList<EmojiAssist.DecoEmojiTextInfo>();

        Cursor cursor = con.query(Const.TABLE_NAME_UNICODE, new String[] {
                Const.COLUMN_HISTORY_ID, Const.COLUMN_CODE }, null, null, null,
                null, Const.ORDER_BY_HISTORY_ID_DESC);

        rowCount = cursor.getCount();

        if (rowCount == 0) {
            cursor.close();

            if (Const.DEBUG) {
                Log.d(Const.TAG, CLASS_NAME + "loadHistories() end. No Histories.");
            }
            return null;
        }

        cursor.moveToFirst();
        for (int i = 0; i < rowCount; i++) {
            EmojiAssist.DecoEmojiTextInfo unicodeEmoji = new EmojiAssist.DecoEmojiTextInfo();
            unicodeEmoji.setUri(cursor.getString(Const.COLUMN_INDEX_CODE));
            mDecoHistories.add(i, unicodeEmoji);
            cursor.moveToNext();
        }
        cursor.close();

        if (Const.DEBUG) {
            Log.d(Const.TAG, CLASS_NAME + "loadHistories() end.");
        }

        return mDecoHistories;
    }

    /**
     * Unicode絵文字入力履歴テーブルを更新する.
     * 
     * @param code
     *            Unicode絵文字の文字コード
     */
    void updateHistoryUnicode(String code) {
        if (Const.DEBUG) {
            Log.d(Const.TAG, CLASS_NAME + "updateHistory() start. code[" + code + "]");
        }

        // 2度押し対策
        if (mDatabase == null) {
            Log.e(Const.TAG, "mDatabase is null.");
            return;
        }

        int i = 0;
        SQLiteDatabase con = mDatabase;

        Cursor cursor = con.query(Const.TABLE_NAME_UNICODE, new String[] {
                Const.COLUMN_HISTORY_ID, Const.COLUMN_CODE }, null, null, null,
                null, Const.ORDER_BY_HISTORY_ID_ASC);

        int historyCount = cursor.getCount();
        if (Const.DB_MAX_HISTORY_RECORD < historyCount) {
            historyCount = Const.DB_MAX_HISTORY_RECORD;
        }

        int lastRowId = 0;

        if (cursor.moveToLast()) {

            // 最終レコードのhistory_idを取得する
            lastRowId = cursor.getInt(Const.COLUMN_INDEX_HISTORY_ID);

            // IDが上限となった時、全レコードを1から採番する
            if (lastRowId == Const.MAX_HISTORY_ID) {

                cursor.moveToFirst();
                ContentValues args = new ContentValues();
                int rowId = 0;

                con.beginTransaction();
                if (Const.DEBUG) {
                    Log.d(Const.TAG, CLASS_NAME
                            + "updateHistory() MaxHistoryId Update Transaction Begin.");
                }

                for (i = 0; i < historyCount; i++) {
                    rowId = cursor.getInt(Const.COLUMN_INDEX_HISTORY_ID);
                    args.put(Const.COLUMN_HISTORY_ID, i);
                    con.update(Const.TABLE_NAME_UNICODE, args,
                            Const.COLUMN_HISTORY_ID + "=" + rowId, null);
                    cursor.moveToNext();
                }

                con.setTransactionSuccessful();
                if (Const.DEBUG) {
                    Log.d(Const.TAG, CLASS_NAME
                            + "updateHistory() MaxHistoryId Update Transaction Successful.");
                }

                con.endTransaction();
                if (Const.DEBUG) {
                    Log.d(Const.TAG, CLASS_NAME
                            + "#updateHistory() MaxHistoryId Update Transaction End.");
                }

                cursor.close();

                cursor = con.query(Const.TABLE_NAME_UNICODE, new String[] {
                        Const.COLUMN_HISTORY_ID, Const.COLUMN_CODE }, null,
                        null, null, null, Const.ORDER_BY_HISTORY_ID_ASC);
                lastRowId = i;
            }
            lastRowId++;
        } else {
            lastRowId = 1;
        }

        // 既に登録されているURIであった時、該当レコードを最終レコードとして更新する
        if (0 < historyCount) {

            int dbHistoryId = 0;
            cursor.moveToFirst();

            // 取得できた件数分、検索を行う
            for (i = 0; i < historyCount; i++) {
                String dbUri = cursor.getString(Const.COLUMN_INDEX_CODE);
                if (dbUri.equalsIgnoreCase(code)) {
                    dbHistoryId = cursor.getInt(Const.COLUMN_INDEX_HISTORY_ID);
                    cursor.close();
                    // 1件目で同一のURIがヒットした時は、更新する必要なし
                    if ((i + 1) != historyCount) {

                        con.beginTransaction();
                        if (Const.DEBUG) {
                            Log.d(Const.TAG, CLASS_NAME
                                    + "updateHistory() IsExistUri Update Transaction Begin.");
                        }

                        ContentValues args = new ContentValues();
                        args.put(Const.COLUMN_HISTORY_ID, lastRowId);
                        con.update(Const.TABLE_NAME_UNICODE, args, Const.COLUMN_HISTORY_ID + "="
                                + dbHistoryId, null);

                        con.setTransactionSuccessful();
                        if (Const.DEBUG) {
                            Log.d(Const.TAG,
                                    CLASS_NAME
                                            + "updateHistory() IsExistUri Update Transaction Successful.");
                        }

                        con.endTransaction();
                        if (Const.DEBUG) {
                            Log.d(Const.TAG, CLASS_NAME
                                    + "updateHistory() IsExistUri Update Transaction End.");
                        }
                    }

                    if (Const.DEBUG) {
                        Log.d(Const.TAG, CLASS_NAME + "updateHistory() end. IsExistUri.");
                    }
                    return;
                }
                cursor.moveToNext();
            }

            // 登録数が上限となった時、最も古いレコードを削除する
            if (i >= Const.DB_MAX_HISTORY_RECORD) {

                con.beginTransaction();
                if (Const.DEBUG) {
                    Log.d(Const.TAG, CLASS_NAME
                            + "updateHistory() MaxHistoryRecord Delete Transaction Begin.");
                }

                cursor.moveToFirst();
                dbHistoryId = cursor.getInt(Const.COLUMN_INDEX_HISTORY_ID);
                con.delete(Const.TABLE_NAME_UNICODE, Const.COLUMN_HISTORY_ID + "=" + dbHistoryId, null);

                con.setTransactionSuccessful();
                if (Const.DEBUG) {
                    Log.d(Const.TAG,
                            CLASS_NAME
                                    + "updateHistory() MaxHistoryRecord Delete Transaction Successful.");
                }

                con.endTransaction();
                if (Const.DEBUG) {
                    Log.d(Const.TAG, CLASS_NAME
                            + "updateHistory() MaxHistoryRecord Delete Transaction End.");
                }
            }
        }
        cursor.close();

        con.beginTransaction();
        if (Const.DEBUG) {
            Log.d(Const.TAG, CLASS_NAME
                    + "updateHistory() NewHistoryRecord Insert Transaction Begin.");
        }

        // 入力履歴を最終レコードに登録する
        ContentValues cv = new ContentValues();
        cv.put(Const.COLUMN_HISTORY_ID, lastRowId);
        cv.put(Const.COLUMN_CODE, code);
        con.insert(Const.TABLE_NAME_UNICODE, null, cv);

        con.setTransactionSuccessful();
        if (Const.DEBUG) {
            Log.d(Const.TAG, CLASS_NAME
                    + "updateHistory() NewHistoryRecord Insert Transaction Successful.");
        }

        con.endTransaction();
        if (Const.DEBUG) {
            Log.d(Const.TAG, CLASS_NAME
                    + "updateHistory() NewHistoryRecord Insert Transaction End.");
        }

        if (Const.DEBUG) {
            Log.d(Const.TAG, CLASS_NAME + "updateHistory() end.");
        }
    }

    /**
     * カテゴリ情報一覧を取得する.
     * 
     * @param DecoEmojiListViewManager
     * @param 表示可能な絵文字種別
     * @return カテゴリ情報一覧のリスト
     */
    ArrayList<DecoEmojiCategoryInfo> getCategoryInfoList(
            DecoEmojiListViewManager listViewManager, int emojiType) {
        if (Const.DEBUG) {
            Log.d(Const.TAG, CLASS_NAME + "getCategoryInfoList() start");
        }

        mCategoryInfoList = new ArrayList<DecoEmojiCategoryInfo>();

        try {
            // デコレーション絵文字情報一覧取得URI
            // パラメータに重複除外を設定
            Uri uri = DecoEmojiContract.CONTENT_DECOINFOLIST_URI
                      .buildUpon()
                      .appendQueryParameter(DecoEmojiContract.QUERY_PARAM_DISTINCT, "true")
                      .build();
            // 絵文字タイプをSQL条件文字列に変換
            String kind = String.valueOf(DecoEmojiContract.KIND_SQ20);
            // 対象絵文字タイプの値を絵文字種別の値に変換する
            kind = DecoEmojiContract.makeStringEmojiKind(emojiType);
            // 条件句を作成
            // 条件として絵文字種別の設定を行う
            String selection = DecoEmojiContract.DecoEmojiInfoColumns.KIND
                    + " IN (" + kind + ")";
            // order句を作成
            // プリセットの順番に取得できるように設定
            String order = DecoEmojiContract.DecoEmojiInfoColumns.CATEGORY_PRESET_ID + " asc";
            // カラムを作成
            // 条件として{カテゴリID、カテゴリ名(日本語)、カテゴリ名(英語)}の設定を行う
            String[] projection = {DecoEmojiContract.DecoEmojiInfoColumns.CATEGORY_ID,
                                   DecoEmojiContract.DecoEmojiInfoColumns.CATEGORY_NAME_JPN,
                                   DecoEmojiContract.DecoEmojiInfoColumns.CATEGORY_NAME_ENG};

            // カテゴリ取得
            Cursor cursor = mLocalContext.getContentResolver().query(uri, projection, selection, null, order);
            if (cursor != null) {
                if (cursor.getCount() > 0) {
                    while (cursor.moveToNext()) {
                        DecoEmojiCategoryInfo values = new DecoEmojiCategoryInfo();
                        int categoryId = cursor.getInt(cursor.getColumnIndex(DecoEmojiContract.DecoEmojiInfoColumns.CATEGORY_ID));
                        values.setCategoryId(categoryId);
                        values.setCategoryName_jpn(cursor.getString(cursor.getColumnIndex(DecoEmojiContract.DecoEmojiInfoColumns.CATEGORY_NAME_JPN)));
                        values.setCategoryName_eng(cursor.getString(cursor.getColumnIndex(DecoEmojiContract.DecoEmojiInfoColumns.CATEGORY_NAME_ENG)));
                        mCategoryInfoList.add(values);
                    }
                }
                cursor.close();
            }
        } catch (SQLiteException e) {
            return new ArrayList<DecoEmojiCategoryInfo>();
        }

        if (Const.DEBUG) {
            Log.d(Const.TAG, CLASS_NAME + "getCategoryInfoList() end.");
        }

        return mCategoryInfoList;
    }

    /**
     * カテゴリ情報から取得できる絵文字情報のリストを取得する.
     * 
     * @param DecoEmojiListViewManager
     * @param emojiType 表示可能な絵文字種別
     * @param targetPage 表示しているページ数
     * @param categoryInfo カテゴリ情報
     * @param start 取得開始位置
     * @param picture デコメピクチャ区分
     *                （true:デコメピクチャ,false：デコメ絵文字）
     * @return 絵文字情報のリスト
     */
    List<EmojiAssist.DecoEmojiTextInfo> getDecoEmojiList(
            DecoEmojiListViewManager listViewManager, int emojiType,
            int targetPage, DecoEmojiCategoryInfo categoryInfo, int start,
            boolean picture) {
        if (Const.DEBUG) {
            Log.d(Const.TAG, CLASS_NAME + "getDecoEmojiList() start");
        }

        mCurrentDecoEmojiList = new ArrayList<EmojiAssist.DecoEmojiTextInfo>();
        mDecoEmojiListCount = 0;

        List<EmojiAssist.DecoEmojiTextInfo> tmpDecoEmojiList = new ArrayList<EmojiAssist.DecoEmojiTextInfo>();

        // 保持しているページ数を超えていない時に取得を行う
        if (Const.CATEGORY_DATA_FIRST_PAGE <= targetPage) {

            // 対象絵文字タイプの値を絵文字種別の値に変換する
            String kind = DecoEmojiContract.makeStringEmojiKind(emojiType);

            try {
                // URIを作成
                Uri uri = DecoEmojiContract.CONTENT_DECOINFOLIST_URI;
                // 条件句を作成
                // 条件として絵文字種別の設定を行う
                String selection[] = {
                        DecoEmojiContract.DecoEmojiInfoColumns.URI,
                        DecoEmojiContract.DecoEmojiInfoColumns.WIDTH,
                        DecoEmojiContract.DecoEmojiInfoColumns.HEIGHT,
                        DecoEmojiContract.DecoEmojiInfoColumns.KIND,
                        DecoEmojiContract.DecoEmojiInfoColumns.DECOME_POP_FLAG,
                        DecoEmojiContract.DecoEmojiInfoColumns.TAGS };
                // カラムを作成
                String where = " (" + DecoEmojiContract.DecoEmojiInfoColumns.CATEGORY_ID + " = ?) and ( "
                                + DecoEmojiContract.DecoEmojiInfoColumns.KIND + " in( " + kind + " ) ) ";
                if (!picture) {
                    // デコメ絵文字のみ取得
                    where = where + "and ( " + DecoEmojiContract.DecoEmojiInfoColumns.KIND
                                  + " <> " + DecoEmojiContract.KIND_PICTURE + " )";
                } else {
                    // デコメピクチャのみ取得
                    where = where + "and ( " + DecoEmojiContract.DecoEmojiInfoColumns.KIND
                                  + " = " + DecoEmojiContract.KIND_PICTURE + " )";
                }
                // order句を作成
                // 表示順で取得できるようにソート条件を設定する
                String order = DecoEmojiContract.DecoEmojiInfoColumns.LAST_USE_CNT + " desc,"
                        + DecoEmojiContract.DecoEmojiInfoColumns.URI + " limit "
                        + listViewManager.getDecoEmojiMaxDrawableCount()
                        + " offset " + String.valueOf(start);

                String arg[] = {String.valueOf(categoryInfo.getCategoryId())};
                // カテゴリ一覧をコンテンツプロバイダから取得する。
                // URI一覧取得
                Cursor cursor = mLocalContext.getContentResolver()
                        .query(uri, selection, where, arg, order);
                if (cursor != null) {
                    while (cursor.moveToNext()) {
                        // デコメピクチャの判定を行い、デコメピクチャは最後に一括で追加するようにする
                        EmojiAssist.DecoEmojiTextInfo decoEmojiTextInfo = new EmojiAssist.DecoEmojiTextInfo();
                        decoEmojiTextInfo.setUri(cursor.getString(0));
                        decoEmojiTextInfo.setWidth(cursor.getInt(1));
                        decoEmojiTextInfo.setHeight(cursor.getInt(2));
                        decoEmojiTextInfo.setKind(cursor.getInt(3));
                        decoEmojiTextInfo.setPop(cursor.getInt(4));
                        if (!cursor.getString(5).equals("")) {
                            decoEmojiTextInfo.setCustomTags(true);
                        }
                        mCurrentDecoEmojiList.add(decoEmojiTextInfo);
                    }
                    cursor.close();
                }
                mCurrentDecoEmojiList.addAll(tmpDecoEmojiList);
                mDecoEmojiListCount = mCurrentDecoEmojiList.size();
            } catch (SQLiteException e) {
                return new ArrayList<EmojiAssist.DecoEmojiTextInfo>();
            }
        }

        if (Const.DEBUG) {
            Log.d(Const.TAG, CLASS_NAME + "getDecoEmojiList() end.");
        }

        return mCurrentDecoEmojiList;
    }

    /**
     * 表示絵文字数を取得する.
     * 
     * @return カテゴリ内に表示する絵文字数
     */
    int getDecoEmojiListCount() {
        return mDecoEmojiListCount;
    }
}
